<?php
	
	/**
	 * USAGE
	 */
	
	/* 
		New Line = \n
		Float Left = @
		Bigger Font = +
		Smaller Font = ~
		Italic = ^
		Bold = *
		Tables = { and }
		Table Cell = []
		Table Row = <>
		Plain Text = `text`
	*/
	
	class TokenFormatter {
		
		private $string;
		private $tokens;
		private $parameters = array();
		private $flags = array();
		private $html;
		
		private $table_css_class = "";
		private $td_css_class = "";
		
		/**
		 * Accessors
		 */
		public function set_table_css_class ($class) {
			$this->table_css_class = $class;
		}
		
		public function set_td_css_class ($class) {
			$this->td_css_class = $class;
		}
		
		/**
		 * Private
		 */
		
		private function parse_parameters ($string) {
			if (preg_match("/(\w{1}):(\d+)/", $string, $matches)) {
				$this->parameters[$matches[1]] = $matches[2];
				return str_replace($matches[1].":".$matches[2], $matches[1], $string);
			} else {
				return $string;
			}
		}
		
		private function apply_formatting (&$output, $element, &$flags) {
			
			// plain text mode
			if (!$this->html) return false;
			
			$formatted = false; 
			
			switch ($element) {

				case '@':
					$output .= "<span style=\"float: left\">";
					$flags["float"] = true;
					$formatted = true;
					break; 

				/*
				case '≥':
					$output .= "<span style=\"float: right\">";
					$flags["float"] = true;
					$formatted = true;
					break; 
				*/

				case '+':
					$output .= "<span style=\"font-size: larger\">";
					$flags["bigger"] += 1;
					$formatted = true;
					break;
				case '~':
					$output .= "<span style=\"font-size: smaller\">";
					$flags["smaller"] -= 1;
					$formatted = true;
					break;
				case '^':
					$output .= "<span style=\"font-style: italic\">";
					$flags["italic"] = true;
					$formatted = true;
					break;
				case '*':
					$output .= "<span style=\"font-weight: bold\">";
					$flags["bold"] = true;
					$formatted = true;
					break;

				case '{':
					$output .= "<table border=\"0\" class=\"$this->table_css_class\">";
					$formatted = true;
					break;
				case '}':
					$output .= "</table>";
					$formatted = true;
					break;

				case '[':
					$output .= "<td valign=\"middle\" class=\"$this->td_css_class\">";
					$formatted = true;
					break;
				case ']':
					$output .= "</td>";
					$formatted = true;
					break;			

				case "<":
					$output .= "<tr>";
					$formatted = true;
					break;
				case ">":
					$output .= "</tr>";
					$formatted = true;
					break;

				/*	
				case '[':
					$output .= "<div class=\"center\">";
					$formatted = true;
					break;
				case ']':
					$output .= "</div>";
					$formatted = true;
					break;
				*/

				case "\n":
					$output .= "<br/>";
					$formatted = true;
					break;
				case ' ':
					$output .= "&nbsp;";
					$formatted = true;
					break;
			}

			return $formatted;
		}

		private function terminate_formating (&$output, &$flags) {
			foreach ($flags as $key => $value) {

				if (is_bool($value)) {
					$output .= "</span>";
					continue;
				}

				if (is_int($value)) {
					for ($i=0; $i < $value; $i++) $output .= "</span>";
					continue;
				}
			}

			$flags = array();
		}		
		
		/**
		 * Public
		 */
		
		public function parse () {
			$output = "";
			
			$this->string = $this->parse_parameters($this->string);
			
			// replace literal \n with space
			$this->string = str_replace('\n', "\n", $this->string);
			//print_r($this->parameters);
			
			$elements = str_split($this->string);
			$user_text = false;
			
			//print_r($elements);
			foreach ($elements as $element) {
				
				// parse user text
				if ($element == '`') {
					if ($user_text) {
						$user_text = false;
					} else {
						$user_text = true;
					}
					continue;
				}
				
				// append user text
				if ($user_text) {
					$output .= $element;
					continue;
				}
				
				if ($this->apply_formatting($output, $element, $this->flags)) continue;
				
				if (array_key_exists($element, $this->tokens)) {
					$value = $this->tokens[$element];
					
					// replace parameter
					for ($i=0; $i < count($this->parameters); $i++) { 
						$value = str_replace("@", $this->parameters[$element], $value);
					}
					
					$output .= $value;
				} else {
					$output .= $element;
				}
				
				$this->terminate_formating($output, $this->flags);
			}
			
			return $output;
		}
		
		function __construct ($string, $tokens, $html = true) {
			$this->string = $string;
			$this->flags = array();
			$this->tokens = $tokens;
			$this->html = $html;
		}
		
	}
	

	function test () {
		$tokens = array(
										// location
										"a" => "apple",
										"b" => "bannana",
										"c" => "carrot",
										"i" => "<img src=\"foo\" width=\"@\" height=\"@\"/>",
										);

		$parser = new TokenFormatter("`text:` i:128+a\n~b\n~c", $tokens);
		$output = $parser->parse();
		print($output);
	}

?>